@@ -48,7 +48,7 @@ class WorkbenchMenu:
         except ValueError:
             return False
 
-    def questionary(self, fname, msg, validate_type=None, **kwargs) -> str:
+    def questionary(self, fname, msg, validate_type=None, output_f=None, **kwargs) -> str:
         """Wrap questionary functions to catch escapes and exit gracefully."""
 
         # Get the questionary function
@@ -77,6 +77,12 @@ class WorkbenchMenu:
         if resp is None:
             self.exit()
 
+        # If an output transformation function was defined
+        if output_f is not None:
+
+            # Call the function
+            resp = output_f(resp)
+
         # Otherwise
         return resp
 
@@ -219,7 +225,7 @@ class WorkbenchMenu:
 
                 # Add option for viewing the logs
                 options.append(
-                    ("View Logs", self.view_logs_menu)
+                    ("View Logs", self.tail_logs)
                 )
 
             # Add option for editing the dataset
@@ -486,10 +492,10 @@ class WorkbenchMenu:
 
         # Present the user with a list of assets and get their response
         # If they want to select none, they can use the "Back" option provided
-        selection = self._prompt_user_to_select_asset(asset_type)
+        repo_name, asset_name = self._prompt_user_to_select_asset(asset_type)
 
         # If the user decided to go back
-        if selection == "Back":
+        if repo_name == "Back" or asset_name == "Back":
 
             # Go back
             self.main_menu()
@@ -497,9 +503,6 @@ class WorkbenchMenu:
         # If the user selected a tool
         else:
 
-            # Parse the repository and asset from the selection
-            repo_name, asset_name = selection.split(": ")[0].split("/", 1)
-
             # Drop the user into the menu used to browse a single asset
             self._browse_single_asset(
                 asset_type=asset_type,
@@ -1111,7 +1114,7 @@ class WorkbenchMenu:
 
         # Set up the text used to prompt the user
         print_logs_prompt = "View more logs"
-        exit_logs_prompt = "Return to main menu (leaving process running)"
+        exit_logs_prompt = "Return to main menu"
 
         # Print logs to start
         user_choice = print_logs_prompt
@@ -1128,35 +1131,45 @@ class WorkbenchMenu:
                 # Refresh the status from the index
                 ds.read_index()
 
-                # If there is a 'status' defined as FAILED or COMPLETED
-                if ds.index.get('status') in ["FAILED", "COMPLETED"]:
+                # Get the list of custom actions which may
+                # optionally be available
+                actions = ds.get_actions()
 
-                    # Then we will exit the logs
-                    user_choice = exit_logs_prompt
-                    sleep(0.5)
+                # Ask the user what they want to do, while also showing
+                # the dataset status
+                user_choice = self.questionary(
+                    "select",
+                    f"Options (status: {ds.index.get('status', 'PENDING')})",
+                    choices=[
+                        print_logs_prompt,
+                        exit_logs_prompt
+                    ] + actions
+                )
 
-                # If the dataset is still running
-                else:
+                # Erase the prompt line
+                self.erase_lines(3)
+                # If the user selected an action
+                if user_choice in actions:
 
-                    # Get the list of custom actions which may
-                    # optionally be available
-                    actions = ds.get_actions()
+                    # Run the action
+                    ds.run_action(user_choice)
 
-                    # Ask the user what they want to do
-                    user_choice = self.questionary(
-                        "select",
-                        "Options",
-                        choices=[
-                            print_logs_prompt,
-                            exit_logs_prompt
-                        ] + actions
-                    )
+    def erase_lines(self, n_lines:int):
+        """Erase a number of lines from the display"""
 
-                    # If the user selected an action
-                    if user_choice in actions:
+        assert isinstance(n_lines, int)
+        assert n_lines > 0
 
-                        # Run the action
-                        ds.run_action(user_choice)
+        # Erase the current line
+        sys.stdout.write('\033[2K\033[1G')
+
+        # For each additional line
+        if n_lines > 1:
+            for _ in range(n_lines - 1):
+                # Go up a line
+                sys.stdout.write("\033[F")
+                # Erase that line
+                sys.stdout.write('\033[2K\033[1G')
 
     def jump_directory_menu(self, sep=" : "):
         """Select an indexed directory and navigate to it."""
@@ -1215,6 +1228,17 @@ class WorkbenchMenu:
             ("Back to Main Menu", self.main_menu)
         ]
 
+        # Make a dict of functions
+        func_map = dict()
+
+        # Make a list of choices to present
+        choices = list()
+
+        # Populate the dictionary and the list
+        for name, f in options:
+            func_map[name] = f
+            choices.append(name)
+
         # Add options to modify each of the downloaded and linked repositories
 
         # Get the list of repositories which have been downloaded
@@ -1222,48 +1246,71 @@ class WorkbenchMenu:
 
         # Add an option for each of the downloaded repositories
         for repo in local_repos:
-            options.append(
-                (
-                    f"Manage repository {repo}",
-                    lambda: self.manage_repo(repo)
-                )
-            )
+            choices.append(f"Manage repository: {repo}")
 
-        # Ask the user what to do
-        self.select_func(
+        # Ask the user
+        choice = self.questionary(
+            "select",
             "Manage Repositories",
-            options
+            choices=choices
         )
 
+        # If the user decided to manage a repository
+        if choice.startswith("Manage repository: "):
+
+            # Get the repository name
+            repo = choice[len("Manage repository: "):]
+
+            # Launch the menu for managing that repository
+            self.manage_repo(repo)
+
+        # If the user selected something else
+        else:
+
+            # Make sure that we know what function to run
+            assert choice in func_map, f"Cannot map selection ({choices}) to options ({', '.join(list(func_map.keys()))})"
+
+            # Open that menu
+            func_map[choice]()
+
     def download_repo(self):
         """Download a GitHub repository."""
 
         # Get the name of the repository to download
         repo_name = self.questionary("text", "Repository name")
 
-        # Make sure that the user has checked the spelling and trusts
-        # the content of this repository
-        prompt = textwrap.dedent(f"""
-        Do you trust the code in this repository?
-        
-        Make sure that the spelling of the repository is correct: {repo_name}
+        # If the user entered an empty string, or one without a '/' in the middle
+        if len(repo_name) < 3 or "/" not in repo_name or len(repo_name.split("/")) != 2:
 
-        Press <ENTER> or Y to confirm download.""")
+            # Tell the user that the repository is not valid
+            self.print_line(f"Repository name not valid: '{repo_name}'")
 
-        # If the user is not sure
-        if not self.questionary("confirm", prompt):
+        # If the repository name is plausible
+        else:
 
-            # Go back to the repository menu
-            self.manage_repositories_menu()
+            # Make sure that the user has checked the spelling and trusts
+            # the content of this repository
+            prompt = textwrap.dedent(f"""
+            Do you trust the code in this repository?
+            
+            Make sure that the spelling of the repository is correct: {repo_name}
 
-        # Try to download it
-        try:
-            self.wb.add_repo(repo_name)
-        except Exception as e:
-            self.print_line(f"ERROR: {str(e)}")
+            Press <ENTER> or Y to confirm download.""")
 
-        # Update the list of Repositories which are available
-        self.wb.repositories = self.wb.setup_repositories()
+            # If the user is not sure
+            if not self.questionary("confirm", prompt):
+
+                # Go back to the repository menu
+                self.manage_repositories_menu()
+
+            # Try to download it
+            try:
+                self.wb.add_repo(repo_name)
+            except Exception as e:
+                self.print_line(f"ERROR: {str(e)}")
+
+            # Update the list of Repositories which are available
+            self.wb.repositories = self.wb.setup_repositories()
 
         # Back to the repository menu
         self.manage_repositories_menu()
@@ -1563,44 +1610,3 @@ class WorkbenchMenu:
 
         # Go back to the main menu
         self.main_menu()
-
-    def view_logs_menu(self):
-        """Provide options to view the logs present in a dataset."""
-
-        # Get the dataset information
-        ds = self.wb.dataset(self.cwd)
-
-        # Build a list of choices
-        choices = [
-            f"View {log_type}"
-            for log_type in ds.log_types()
-        ] + [
-            "Back"
-        ]
-
-        # Get the user's selection
-        selection = self.questionary(
-            "select",
-            "Select type of logs to display",
-            choices=choices
-        )
-
-        # If they are done
-        if selection == "Back":
-
-            # Go back to the main menu
-            self.main_menu()
-
-        # Otherwise
-        else:
-
-            # Get the log type
-            log_type = selection.split(" ", 1)[1]
-
-            # Print the log type
-            print(
-                ds.read_logs(log_type)
-            )
-
-            # Re-display the same menu
-            self.view_logs_menu()
